//==============================================================================
//File name:   "servo.cpp"
//Purpose:      Main Source File
//Version:      1.0, Example for CAN Servo Gear remote control 
//Copyright:    (c) 2023, Akimov Vladimir  E-mail: decoder@rambler.ru		
//==============================================================================
#include <windows.h>
#include "stdafx.h"
#include <iostream>
#include <conio.h> 
#include "help_msg.h"
#include "getopt.h"
#include "candll.h"
#include "functions.h"

//Using Dll
#ifdef _WIN64
  #pragma comment(lib,"./src/CAN/x64/candll64.lib")//For 64bit Win
#else
  #pragma comment(lib,"./src/CAN/x86/candll.lib") //For 32bit Win
#endif

using namespace std;

//------------------------------------------------
//Environment->PATH=$(ProjectDir)\src\CAN\x86;$(PATH);
//Debug arguments:
//------------------------------------------------
//-a CAN3 -d 0xA1 -c 1 -v 40  +
//-a CAN3 -d 0xA1 -c 1 -v -40 +
//-a CAN3 -d 0xA1 -t +
//-s -u

//=====================================================================
//main
//===================================================================== 
int _tmain(int argc, char *argv[])
{ 
  int opt, device, ret, test=0, command=0, setup=0, update=0;
  signed short  value;
  unsigned long buffer;
  HANDLE pCAN = NULL;
  unsigned char CAN_device_ID, channel;
  char adapter[5], txt_cmd[32];
  
  //clear buffers
  memset(adapter, 0, sizeof(adapter));
  memset(txt_cmd, 0, sizeof(txt_cmd));

  printf("Servo gear control, ver 1.0, Build: %s\n",__DATE__);
      
  if(argv[1]==NULL){PrintUsage(); return 1;}
 
  //debug 
  print_arg(argc, argv);
 
  //options parser
  while((opt = getopt(argc, argv, "a:d:c:v:tsuh"))!= EOF) 
  {
     switch(opt) 
        {
        //set usb-can adapter name
        case 'a':
            if(strlen(optarg)>5){printf("\na-arg incorrect"); return 1;}
            memcpy(adapter, optarg, strlen(optarg));
            break;

        //set can device id number
        case 'd':
			//CAN_device_ID = (unsigned char) atoi(optarg);
			ret = sscanf_s(optarg, "%i", &buffer);
			CAN_device_ID = (unsigned char) buffer;
            if(ret!=1 && (CAN_device_ID<1 || CAN_device_ID>254))
			  {printf("\nd-arg incorrect"); return 1;}
            break;

		//set servo channel number 
        case 'c':
			channel = atoi(optarg);
            if(channel<1 || channel>2)
			  {printf("\nc-arg incorrect"); return 1;}		
            break;

        //set gear angle value 
        case 'v':
			command = 1;
			value = atoi(optarg);
            //if(channel<1 || channel>2)
			//  {printf("\nc-arg incorrect"); return 1;}
            break;
        
		//execute test 
        case 't':	
		    test = 1;
            break;		
		
	    //call CAN adapter setup dialog 
        case 's':	
		    setup = 1;
			memcpy(adapter, "CAN", 3);
            break;			
	    
		//call remote device firmware update dialog 
        case 'u':	
		    update = 1;
			memcpy(adapter, "CAN", 3);
            break;

		//show help
        case 'h':
        default:
            //If absent command arg parameters
            PrintUsage();
            return 1;
        }
    }

  //==================================================================
  //Open CAN dll communication
  int result = CAN_Open(&pCAN, adapter, 500); 

  //Check 
  if(result<1 || pCAN==NULL)
   { printf("\nCAN Open error. Exit.\n"); 
     return 1;
   }

  //==================================================================
  //Dialog handlers
  if(setup)
  {	 //  HWND GetConsoleHwnd(void)
     //Communication setup dialog
	 CAN_Setup_Dialog(NULL, pCAN);
  }
  if(update)
  {
  	 //Remote device firmware update dialog
	 CAN_ISP_Dialog(NULL, pCAN);
  }

  //==================================================================
  //Check answer from remote device controller
  device = CAN_CheckDevice(pCAN, CAN_device_ID);

  //Check device online
  if(device!=1)
   {
     printf("\nCan`t find the device: 0x%X\n", CAN_device_ID); 
     goto close;
   }
            
  //==================================================================
  //Send command/test to remote device
  if(command) SendCommand(pCAN, channel, CAN_device_ID, value);
  if(test) SendTest(pCAN, CAN_device_ID);

  //==================================================================
  //Close dll and communication
  close:
  CAN_Close(pCAN);
  
  return 0;
}
