//==============================================================================
//File name:    "functions.cpp"
//Purpose:      Source File
//Version:      1.00
//Copyright:    (c) 2023, Akimov Vladimir  E-mail: decoder@rambler.ru	
//==============================================================================
#include <windows.h>
#include <iostream>
#include <conio.h> 
#include "help_msg.h"
#include "getopt.h"
#include "candll.h"
#include "functions.h"

#define CMD_SET_ANGLE_CH2	0xFB
#define CMD_SET_ANGLE_CH1	0xFA

//------------------------------------------------------------------------------
//Function:
//------------------------------------------------------------------------------
void SendTest(HANDLE hCAN, int device_id)
{
  printf("\nSend test 3 cmd sequence.\n");

  //Send command 1 to remote device
  SendCommand(hCAN, 1, device_id, 0);
  Sleep(1000);
  
  //Send command 2 to remote device
  SendCommand(hCAN, 1, device_id, -80);
  Sleep(1000);
  
  //Send command 3 to remote device
  SendCommand(hCAN, 1, device_id, 80);
  Sleep(1000);
}

//------------------------------------------------------------------------------
//Debug	Function:
//------------------------------------------------------------------------------
void print_arg(int argc, char *argv[])
{
  printf("\nargc = %d", argc);

  int i = 1;
  do
  {
    printf("\nargv[%d]= %s",i, argv[i]);
    i++;
  } while (i < argc);
  
  printf("\n\n");
}
//------------------------------------------------------------------------------
//Function:
//------------------------------------------------------------------------------
float CalcAngle(int steps)
{
  float angle =(float)steps;
  angle /=4;

  return angle;
}
//------------------------------------------------------------------------------
//Function:
//------------------------------------------------------------------------------
void SendCommand(HANDLE hCAN, int channel, int device_id, int value)
{
  unsigned char cmd;
  if(channel==1) cmd = CMD_SET_ANGLE_CH1;
  else if(channel==2) cmd = CMD_SET_ANGLE_CH2;  
  else return;

  //----
  signed short steps = (signed short)value;
     
  float angle = CalcAngle(steps);
  printf("Gear turn on: %3.1f degrees\n", angle);

  //DLL, send command 
  CAN_SendCommand(hCAN, device_id, cmd,
                 (unsigned char)steps,(unsigned char)(steps>>8));
  
  //Wait while CAN send data
  Sleep(20);
}
